@@ -114,6 +114,7 @@ class Player:
             self.achievements.add('defeat_zombie')
           if isinstance(obj, Cow) and obj.health <= 0:
             self.health = min(self.health + 1, self._max_health)
+            self._hunger = 0
             self.achievements.add('find_food')
           return
       pickaxe = max(
@@ -179,10 +180,12 @@ class Player:
         self.achievements.add('make_stone_pickaxe')
     if action == 11:  # make iron pickaxe
       wood = self.inventory['wood']
+      coal = self.inventory['coal']
       iron = self.inventory['iron']
-      if wood and iron and furnace:
+      if wood > 0 and coal > 0 and iron > 0 and table and furnace:
         self.inventory['wood'] -= 1
-        self.inventory['stone'] -= 1
+        self.inventory['coal'] -= 1
+        self.inventory['iron'] -= 1
         self.inventory['iron_pickaxe'] += 1
         self.achievements.add('make_iron_pickaxe')
 
@@ -201,10 +204,9 @@ class Cow:
   def update(self, terrain, objects, action):
     if self.health <= 0:
       del objects[objects.index(self)]
-    if self._random.uniform() > 0.5:
-      direction = (0, self._random.randint(-1, 2))
-    else:
-      direction = (self._random.randint(-1, 2), 0)
+    if self._random.uniform() < 0.5:
+      return
+    direction = _random_direction(self._random)
     x = self.pos[0] + direction[0]
     y = self.pos[1] + direction[1]
     if _is_free((x, y), terrain, objects):
@@ -239,15 +241,14 @@ class Zombie:
     if dist <= 4:
       xdist = abs(self.pos[0] - player.pos[0])
       ydist = abs(self.pos[1] - player.pos[1])
-      if xdist > ydist and self._random.uniform() < 0.7:
+      if self._random.uniform() < 0.2:
+        direction = _random_direction(self._random)
+      elif xdist > ydist and self._random.uniform() < 0.7:
         direction = (-np.sign(self.pos[0] - player.pos[0]), 0)
       else:
         direction = (0, -np.sign(self.pos[1] - player.pos[1]))
     else:
-      if self._random.uniform() > 0.5:
-        direction = (0, self._random.randint(-1, 2))
-      else:
-        direction = (self._random.randint(-1, 2), 0)
+      direction = _random_direction(self._random)
     x = self.pos[0] + direction[0]
     y = self.pos[1] + direction[1]
     if _is_free((x, y), terrain, objects):
@@ -257,7 +258,7 @@ class Zombie:
 class Env:
 
   def __init__(
-      self, area=(64, 64), view=4, size=64, length=10000, health=10,
+      self, area=(64, 64), view=4, size=64, length=10000, health=5,
       seed=None):
     self._area = area
     self._view = view
@@ -275,6 +276,7 @@ class Env:
     self._objects = None
     self._simplex = None
     self._achievements = None
+    self._last_health = None
 
   @property
   def observation_space(self):
@@ -296,13 +298,15 @@ class Env:
         'make_wood_pickaxe', 'make_stone_pickaxe', 'make_iron_pickaxe',
     ]
 
-  def _noise(self, x, y, z, sizes):
+  def _noise(self, x, y, z, sizes, normalize=True):
     if not isinstance(sizes, dict):
-      sizes = {1: sizes}
+      sizes = {sizes: 1}
     value = 0
-    for weight, size in sizes.items():
+    for size, weight in sizes.items():
       value += weight * self._simplex.noise3d(x / size, y / size, z)
-    return value / sum(sizes.keys())
+    if normalize:
+      value /= sum(sizes.values())
+    return value
 
   def reset(self):
     self._step = 0
@@ -321,7 +325,8 @@ class Env:
         start = 4 - np.sqrt((x - center[0]) ** 2 + (y - center[1]) ** 2)
         start += 2 * simplex(x, y, 8, 3)
         start = 1 / (1 + np.exp(-start))
-        mountain = simplex(x, y, 0, {1: 15, 0.3: 5}) - 3 * start
+        mountain = simplex(x, y, 0, {15: 1, 5: 0.3}) - 3 * start - 0.03
+        water = simplex(x, y, 3, {15: 1, 5: 0.15}, False) - 2 * start + 0.1
         if start > 0.5:
           self._terrain[x, y] = MATERIAL_IDS['grass']
         elif mountain > 0.15:
@@ -331,35 +336,38 @@ class Env:
             self._terrain[x, y] = MATERIAL_IDS['path']
           elif simplex(x / 5, 2 * y, 7, 3) > 0.4:  # vertical tunnle
             self._terrain[x, y] = MATERIAL_IDS['path']
-          elif simplex(x, y, 1, 8) > 0 and uniform() > 0.8:
+          elif simplex(x, y, 1, 8) > 0 and uniform() > 0.85:
             self._terrain[x, y] = MATERIAL_IDS['coal']
-          elif simplex(x, y, 2, 6) > 0.3 and uniform() > 0.6:
+          elif simplex(x, y, 2, 6) > 0.4 and uniform() > 0.75:
             self._terrain[x, y] = MATERIAL_IDS['iron']
-          elif mountain > 0.25 and uniform() > 0.99:
+          elif mountain > 0.18 and uniform() > 0.995:
             self._terrain[x, y] = MATERIAL_IDS['diamond']
           elif mountain > 0.3 and simplex(x, y, 6, 5) > 0.4:
             self._terrain[x, y] = MATERIAL_IDS['lava']
           else:
             self._terrain[x, y] = MATERIAL_IDS['stone']
-        elif 0.25 < simplex(x, y, 3, 15) <= 0.35 and simplex(x, y, 4, 9) > -0.2:
+        elif 0.25 < water <= 0.35 and simplex(x, y, 4, 9) > -0.2:
           self._terrain[x, y] = MATERIAL_IDS['sand']
-        elif simplex(x, y, 3, 15) > 0.3:
+        elif 0.3 < water:
           self._terrain[x, y] = MATERIAL_IDS['water']
-        else:  # grass
+        else:  # grassland
           if simplex(x, y, 5, 7) > 0 and uniform() > 0.8:
             self._terrain[x, y] = MATERIAL_IDS['tree']
           else:
             self._terrain[x, y] = MATERIAL_IDS['grass']
 
     self._player = Player(center, self._health)
+    self._last_health = self._health
     self._achievements = self._player.achievements.copy()
     self._objects = [self._player]
     for x in range(self._area[0]):
       for y in range(self._area[1]):
+        dist = np.sqrt((x - center[0]) ** 2 + (y - center[1]) ** 2)
         if self._terrain[x, y] in WALKABLE:
-          if self._terrain[x, y] == MATERIAL_IDS['grass'] and uniform() > 0.99:
+          grass = self._terrain[x, y] == MATERIAL_IDS['grass']
+          if dist > 3 and grass and uniform() > 0.98:
             self._objects.append(Cow((x, y), self._random))
-          elif uniform() > 0.993:
+          elif dist > 6 and uniform() > 0.993:
             self._objects.append(Zombie((x, y), self._random))
 
     return self._obs()
@@ -373,10 +381,16 @@ class Env:
     if len(self._player.achievements) > len(self._achievements):
       reward = 1.0
       self._achievements = self._player.achievements.copy()
+    # if self._player.health < self._last_health:
+    #   # Reported returns wouldn't be number of achievements anymore.
+    #   reward = -1.0
     dead = self._player.health <= 0
     over = self._length and self._step >= self._length
     done = dead or over
-    info = {'achievements': self._achievements.copy()}
+    info = {
+        'achievements': self._achievements.copy(),
+        'discount': 1 - float(dead),
+    }
     return obs, reward, done, info
 
   def render(self):
@@ -401,9 +415,10 @@ class Env:
     return canvas.transpose((1, 0, 2))
 
   def _obs(self):
-    obs = {'image': self.render(), 'health': self._player.health}
+    obs = {'image': self.render()}
+    obs['health'] = np.clip(self._player.health, 0, 255).astype(np.uint8)
     for key, value in self._player.inventory.items():
-      obs[key] = np.clip(value, 0, 255)
+      obs[key] = np.clip(value, 0, 255).astype(np.uint8)
     return obs
 
   def _draw(self, canvas, pos, texture):
@@ -444,3 +459,10 @@ def _is_free(pos, terrain, objects, valid=WALKABLE):
   if terrain[pos] not in valid: return False
   if any(obj.pos == pos for obj in objects): return False
   return True
+
+
+def _random_direction(random):
+  if random.uniform() > 0.5:
+    return (0, random.randint(-1, 2))
+  else:
+    return (random.randint(-1, 2), 0)
